package models

import (
	"errors"
	"fmt"
	"reflect"
	"strings"

	"github.com/astaxie/beego/orm"
)

type Ask struct {
	Id                  int64 `orm:"pk"`
	OriginId            string `orm:"size(128)"`
	OriginUrl           string `orm:"size(128)"`
	OriginUid           string `orm:"size(128)"`
	Source              string `orm:"size(128)"`
	Title               string `orm:"size(128)"`
	Content             string `orm:"longtext"`
	AnswerNum           int 
	NickName            string `orm:"size(128)"`
	Gender              string `orm:"size(128)"`
	EducationBackground string `orm:"size(128)"`
	Age                 string `orm:"size(128)"`
	Area                string `orm:"size(128)"`
	Head                string `orm:"size(128)"`
	Marriage            string `orm:"size(128)"`
	CreateTime          string `orm:"size(128)"`
}

func init() {
	orm.RegisterModel(new(Ask))
}

// AddAsk insert a new Ask into database and returns
// last inserted Id on success.
func AddAsk(m *Ask) (id int64, err error) {
	o := orm.NewOrm()
	id, err = o.Insert(m)
	return
}

// GetAskById retrieves Ask by Id. Returns error if
// Id doesn't exist
func GetAskById(id int64) (v *Ask, err error) {
	o := orm.NewOrm()
	v = &Ask{Id: id}
	if err = o.Read(v); err == nil {
		return v, nil
	}
	return nil, err
}

// GetAllAsk retrieves all Ask matches certain condition. Returns empty list if
// no records exist
func GetAllAsk(query map[string]string, fields []string, sortby []string, order []string,
	offset int64, limit int64) (ml []interface{}, err error) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(Ask))
	// query k=v
	for k, v := range query {
		// rewrite dot-notation to Object__Attribute
		k = strings.Replace(k, ".", "__", -1)
		qs = qs.Filter(k, v)
	}
	// order by:
	var sortFields []string
	if len(sortby) != 0 {
		if len(sortby) == len(order) {
			// 1) for each sort field, there is an associated order
			for i, v := range sortby {
				orderby := ""
				if order[i] == "desc" {
					orderby = "-" + v
				} else if order[i] == "asc" {
					orderby = v
				} else {
					return nil, errors.New("Error: Invalid order. Must be either [asc|desc]")
				}
				sortFields = append(sortFields, orderby)
			}
			qs = qs.OrderBy(sortFields...)
		} else if len(sortby) != len(order) && len(order) == 1 {
			// 2) there is exactly one order, all the sorted fields will be sorted by this order
			for _, v := range sortby {
				orderby := ""
				if order[0] == "desc" {
					orderby = "-" + v
				} else if order[0] == "asc" {
					orderby = v
				} else {
					return nil, errors.New("Error: Invalid order. Must be either [asc|desc]")
				}
				sortFields = append(sortFields, orderby)
			}
		} else if len(sortby) != len(order) && len(order) != 1 {
			return nil, errors.New("Error: 'sortby', 'order' sizes mismatch or 'order' size is not 1")
		}
	} else {
		if len(order) != 0 {
			return nil, errors.New("Error: unused 'order' fields")
		}
	}

	var l []Ask
	qs = qs.OrderBy(sortFields...)
	if _, err := qs.Limit(limit, offset).All(&l, fields...); err == nil {
		if len(fields) == 0 {
			for _, v := range l {
				ml = append(ml, v)
			}
		} else {
			// trim unused fields
			for _, v := range l {
				m := make(map[string]interface{})
				val := reflect.ValueOf(v)
				for _, fname := range fields {
					m[fname] = val.FieldByName(fname).Interface()
				}
				ml = append(ml, m)
			}
		}
		return ml, nil
	}
	return nil, err
}

// UpdateAsk updates Ask by Id and returns error if
// the record to be updated doesn't exist
func UpdateAskById(m *Ask) (err error) {
	o := orm.NewOrm()
	v := Ask{Id: m.Id}
	// ascertain id exists in the database
	if err = o.Read(&v); err == nil {
		var num int64
		if num, err = o.Update(m); err == nil {
			fmt.Println("Number of records updated in database:", num)
		}
	}
	return
}

// DeleteAsk deletes Ask by Id and returns error if
// the record to be deleted doesn't exist
func DeleteAsk(id int64) (err error) {
	o := orm.NewOrm()
	v := Ask{Id: id}
	// ascertain id exists in the database
	if err = o.Read(&v); err == nil {
		var num int64
		if num, err = o.Delete(&Ask{Id: id}); err == nil {
			fmt.Println("Number of records deleted in database:", num)
		}
	}
	return
}
